"use strict";

var Annotation = require("./Annotation");

/**
 * Class representing BioEntity.
 * 
 * @param javaObject
 *          object de-serialized ajax query to the server side
 */
function BioEntity() {
}

BioEntity.prototype.getLinkedSubmodelId = function() {
  return this._linkedSubmodelId;
};

BioEntity.prototype.setLinkedSubmodelId = function(linkedSubmodelId) {
  this._linkedSubmodelId = linkedSubmodelId;
};

/**
 * Returns identifier of the BioEntity.
 * 
 * @returns {number} identifier of the BioEntity
 */
BioEntity.prototype.getId = function() {
  return this.id;
};

/**
 *
 * @param {number} id
 */
BioEntity.prototype.setId = function(id) {
  this.id = id;
};

/**
 * Returns model identifier where {@link BioEntity} is located.
 * 
 * @returns {number} model identifier where {@link BioEntity} is located
 */
BioEntity.prototype.getModelId = function() {
  return this._modelId;
};

BioEntity.prototype.setModelId = function(modelId) {
  this._modelId = modelId;
};

BioEntity.prototype.isComplete = function() {
  return this._complete;
};

BioEntity.prototype.setIsComplete = function(complete) {
  this._complete = complete;
};

BioEntity.prototype.getSymbol = function() {
  return this.symbol;
};

BioEntity.prototype.setSymbol = function(symbol) {
  this.symbol = symbol;
};

BioEntity.prototype.getAbbreviation = function() {
  return this._abbreviation;
};

BioEntity.prototype.setAbbreviation = function(abbreviation) {
  this._abbreviation = abbreviation;
};

BioEntity.prototype.getFormula = function() {
  return this._formula;
};

BioEntity.prototype.setFormula = function(formula) {
  this._formula = formula;
};

BioEntity.prototype.setSynonyms = function(synonyms) {
  this._synonyms = synonyms;
};

BioEntity.prototype.getSynonyms = function() {
  return this._synonyms;
};

BioEntity.prototype.setDescription = function(description) {
  this._description = description;
};

BioEntity.prototype.getDescription = function() {
  return this._description;
};

BioEntity.prototype.setType = function(type) {
  if (type === undefined) {
    throw new Error("type cannot be undefined");
  }
  this._type = type;
};

BioEntity.prototype.getType = function() {
  return this._type;
};

BioEntity.prototype.getOther = function(type) {
  if (this._other !== undefined) {
    return (type === undefined) ? this._other : this._other[type];
  }
};

BioEntity.prototype.setOther = function(other) {
  this._other = other;
};

BioEntity.prototype.getHierarchyVisibilityLevel = function() {
  return this._hierarchyVisibilityLevel;
};

BioEntity.prototype.setHierarchyVisibilityLevel = function(hierarchyVisibilityLevel) {
  this._hierarchyVisibilityLevel = hierarchyVisibilityLevel;
};

BioEntity.prototype.getReferences = function() {
  return this.references;
};

BioEntity.prototype.setReferences = function(references) {
  if (references=== undefined) {
    throw new Error("references must be defined");
  }
  this.references = [];
  for (var i = 0; i < references.length; i++) {
    this.references.push(new Annotation(references[i]));
  }
};

/**
 * @returns {Point}
 */
BioEntity.prototype.getCenter = function() {
  throw new Error("Not implemented");
};

module.exports = BioEntity;
